
/* $Id: pd.c,v 1.6 2008/03/11 22:42:27 agraef Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef _MSC_VER
#include <unistd.h>
#endif
#include <pthread.h>
#include <libq.h>
#include "m_pd.h"

MODULE(pd)

DESTRUCTOR(pd,PdObj,ptr)
{
}

typedef struct _q {
  t_object x_obj;
  pthread_mutex_t mut_init;
  pthread_cond_t cond_init;
  int init;               /* set when initialized */
  int n_in, n_out;        /* number of extra inlets and outlets */
  struct _px **in;	  /* extra inlet proxies, see t_px below */
  t_outlet **out;	  /* outlets */
  expr foo;		  /* the object function */
  char *args;		  /* creation arguments */
  char *dir, *script;	  /* patch dir and script name ("q" receiver only) */
  struct _q *next, *prev; /* double-linked list of all q objects */
  expr *xv;               /* expression buffer for asynchronous messaging */
  int a_xv, n_xv;	  /* allocated and current buffer size */
  t_clock *x_clock;       /* wakeup for asynchronous processing */
  pthread_mutex_t mut_x;  /* mutex for protecting access to internal state */
  pthread_mutex_t mut_time;
  pthread_mutex_t mut_wait;
  pthread_cond_t cond_wait;
  double current_time;	  /* current (logical) Pd time */
  double wait_time;	  /* wait for specified time (pd_wait) */
  int wait_set;		  /* indicates that a new pd_wait is pending */
  t_clock *wait_clock;    /* wakeup for pending pd_wait */
} t_q;

typedef struct _px {
  t_object obj;
  t_q *x;		  /* parent */
  int ix;		  /* inlet index */
} t_px;

typedef void (*cleanup_handler)(void*);

/* maximum buffer size for messages */
#define MAXSIZE 4096

FUNCTION(pd,pd_send,argc,argv)
{
  t_q *x;
  if (argc == 2 && isobj(argv[0], type(PdObj), (void**)&x)) {
    expr y = argv[1];
    int n;
    /* wait until the object has been initialized */
    pthread_cleanup_push((cleanup_handler)pthread_mutex_unlock, &x->mut_init);
    pthread_mutex_lock(&x->mut_init);
    while (!x->init)
      pthread_cond_wait(&x->cond_init, &x->mut_init);
    pthread_cleanup_pop(1);
    newref(y);
    release_lock();
    pthread_mutex_lock(&x->mut_x);
    n = x->n_xv;
    if (n+16 <= MAXSIZE && n >= x->a_xv) {
      expr *xv = realloc(x->xv, (n+16)*sizeof(expr));
      if (xv) {
	x->a_xv += 16;
	x->xv = xv;
      }
    }
    if (n < x->a_xv) {
      x->xv[x->n_xv++] = y;
      pthread_mutex_unlock(&x->mut_x);
      acquire_lock();
      return mkvoid;
    } else {
      pthread_mutex_unlock(&x->mut_x);
      fprintf(stderr, "pd_send(%s): buffer overflow (runaway thread?)\n",
	      x->args);
      acquire_lock();
      unref(y);
      return __ERROR;
    }
  } else
    return __FAIL;
}

FUNCTION(pd,pd_time,argc,argv)
{
  t_q *x;
  if (argc == 1 && isobj(argv[0], type(PdObj), (void**)&x)) {
    double t;
    pthread_mutex_lock(&x->mut_time);
    t = x->current_time;
    pthread_mutex_unlock(&x->mut_time);
    return mkfloat(t);
  } else
    return __FAIL;
}

FUNCTION(pd,pd_wait,argc,argv)
{
  t_q *x;
  double t;
  if (argc == 2 && isobj(argv[0], type(PdObj), (void**)&x) &&
      (isfloat(argv[1], &t) || ismpz_float(argv[1], &t))) {
    pthread_cleanup_push((cleanup_handler)pthread_mutex_unlock, &x->mut_wait);
    pthread_mutex_lock(&x->mut_wait);
    release_lock();
    x->wait_time = t;
    x->wait_set = 1;
    while (x->wait_time > 0.0)
      pthread_cond_wait(&x->cond_wait, &x->mut_wait);
    pthread_cleanup_pop(1);
    acquire_lock();
    return mkvoid;
  } else
    return __FAIL;
}
