import pd, dict;
from pd import list;

/* Controller map for freeax.dsp for use with Oxygen8 keyboard.
   Loaded by freeax-oxygen8.pd

   NOTE: After loading freeax-oxygen8.pd in Pd, it is necessary
   to click on the 

  Map MIDI controller numbers and values to the corresponding
  Faust control names and values. E.g., the entry 1,"myfaustctrl#1",\X.X/127
  maps MIDI controller #1 to Faust control "myfaustctrl#1", dividing
  controller values by 127 to give a normalized value in the range 0..1. */

/* Oxygen8 knob default controller assignments:

       74 71 81 91
       16 80 19  2

   Our assignements here:

       Bright 	 Posit	Drive	FBGain
       Decay	 Detun	VibFrq	FBRes1
 */

def MAP = [
 128,  "pitchbend-semitones",	\X.(X-8192)/8192; // pitch bend -> -1..1
 1,    "vibrato-percent",      \X.X*10/127;	// 0..127 -> 0..10
 7,    "level-linear",	       \X.X/127;	// 0..127 -> 0..1
 74,   "brightness",		\X.X/127;	// 0..127 -> 0..1
 16,   "decaytime-T60",		\X.X*10/127;	// 0..127 -> 0..10
 71,   "excitation-position",	\X.X*0.5/127;	// 0..127 -> 0..0.5
 80,   "coupling-detuning-percent", \X.X*10/127; // 0..127 -> 0..10
 81,   "distortion-drive", 	\X.X/127;	// 0..127 -> 0..1
 19,   "vibrato-freq-Hz", 	\X.X*10/127;	// 0..127 -> 0..10
 91,   "Amp-feedback-gain", 	\X.X*0.01/127;	// 0..127 -> 0..0.01
  2,   "Amp-feedback-first-resonance-Hz", \X.X*2000/127; // 0..127 -> 0..2000
], (NUMS,NAMES,FUNCS) = unzip3 MAP, CD = dict $ zip NUMS $ zip NAMES FUNCS;

/* A simple Pd object which processes MIDI note messages on its left inlet and
  MIDI controller messages on its right inlet, and outputs the translated
  messages on its left and right right outlets. Expects the packed output of
  the Pd [notein] and [ctlin] objects on the left and right inlet,
  respectively. All output messages have the MIDI channel number prepended so
  that they can easily be routed to the corresponding synth unit. The output
  note messages have the format [1 note# gain] with gain=velocity/127, so
  that they are ready to be sent to the first inlet of a monophonic synth
  unit. Similarly, the output control messages have the format
  [faust-control-name mapped-value] which can be sent to the right inlet of a
  synth unit. */

controls        = (process,2,2);

process (0,list N V K)
               = (0,list K 1 N (V/127));
process (1,list V N K)
               = (1,list K NAME X)
                   where (NAME,FUNC) = CD!N, X:Num = FUNC V;

/* Dummy "synth" unit for testing purposes. Prints received note and
  controller messages. See test.pd. You'll have to replace these with your
  string synths. */

mysynth K       = (dummy K,2,0);

dummy K (0,list 1 N V)
               = printf "[%d] note: %d %g\n" (K,N,V);
dummy K (1,NAME VAL)
               = printf "[%d] ctrl: %s %g\n" (K,str NAME,VAL);
